@@ -11,6 +11,8 @@ module Jabber
   # which is used for all messaging communication.
   class Message < XMPPStanza
 
+    CHAT_STATES = %w(active composing gone inactive paused).freeze
+
     name_xmlns 'message', 'jabber:client'
     force_xmlns true
 
@@ -26,7 +28,7 @@ module Jabber
         set_to(to)
       end
       if !body.nil?
-        add_element(REXML::Element::new("body").add_text(body))
+        add_element(REXML::Element.new("body").add_text(body))
       end
     end
 
@@ -144,5 +146,35 @@ module Jabber
     def thread
       first_element_text('thread')
     end
+
+    ##
+    # Returns the current chat state, or nil if no chat state is set
+    def chat_state
+      each_elements(*CHAT_STATES) { |el| return el.name.to_sym }
+      return nil
+    end
+
+    ##
+    # Sets the chat state :active, :composing, :gone, :inactive, :paused
+    def chat_state=(s)
+      s = s.to_s
+      raise InvalidChatState, "Chat state must be one of #{CHAT_STATES.join(', ')}" unless CHAT_STATES.include?(s)
+      CHAT_STATES.each { |state| delete_elements(state) }
+      add_element(REXML::Element.new(s).add_namespace('http://jabber.org/protocol/chatstates'))
+    end
+
+    ##
+    # Sets the message's chat state
+    def set_chat_state(s)
+      self.state = s
+      self
+    end
+
+    CHAT_STATES.each do |state|
+      define_method("#{state}?") do
+        chat_state == state.to_sym
+      end
+    end
+
   end
 end
